# coding: utf-8

"""
    Environmental Exposures API

    Environmental Exposures API

    OpenAPI spec version: 1.0.0
    Contact: stealey@renci.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from __future__ import absolute_import

import sys
import os
import re

# python 2 and python 3 compatibility library
from six import iteritems

from ..configuration import Configuration
from ..api_client import ApiClient


class DefaultApi(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        config = Configuration()
        if api_client:
            self.api_client = api_client
        else:
            if not config.api_client:
                config.api_client = ApiClient()
            self.api_client = config.api_client

    def exposures_exposure_type_coordinates_get(self, exposure_type, **kwargs):
        """
        Get exposure location(s) as latitude, longitude coordinates
        Returns paginated list of available latitude, longitude coordinates for given exposure_type. Optionally the user can provide a latitude, longitude coordinate with a radius in meters to discover if an exposure location is within the requested range

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_exposure_type_coordinates_get(exposure_type, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str exposure_type: The name of the exposure factor (currently limited to pm25, o3). (required)
        :param str latitude: Search coordinates that match or are like 'latitude'
        :param str longitude: Search coordinates that match or are like 'longitude'
        :param str radius: radius in meters to search within for exposure point when a coordinate is provided
        :return: list[Coordinate]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.exposures_exposure_type_coordinates_get_with_http_info(exposure_type, **kwargs)
        else:
            (data) = self.exposures_exposure_type_coordinates_get_with_http_info(exposure_type, **kwargs)
            return data

    def exposures_exposure_type_coordinates_get_with_http_info(self, exposure_type, **kwargs):
        """
        Get exposure location(s) as latitude, longitude coordinates
        Returns paginated list of available latitude, longitude coordinates for given exposure_type. Optionally the user can provide a latitude, longitude coordinate with a radius in meters to discover if an exposure location is within the requested range

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_exposure_type_coordinates_get_with_http_info(exposure_type, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str exposure_type: The name of the exposure factor (currently limited to pm25, o3). (required)
        :param str latitude: Search coordinates that match or are like 'latitude'
        :param str longitude: Search coordinates that match or are like 'longitude'
        :param str radius: radius in meters to search within for exposure point when a coordinate is provided
        :return: list[Coordinate]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['exposure_type', 'latitude', 'longitude', 'radius']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method exposures_exposure_type_coordinates_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'exposure_type' is set
        if ('exposure_type' not in params) or (params['exposure_type'] is None):
            raise ValueError("Missing the required parameter `exposure_type` when calling `exposures_exposure_type_coordinates_get`")


        collection_formats = {}

        resource_path = '/exposures/{exposure_type}/coordinates'.replace('{format}', 'json')
        path_params = {}
        if 'exposure_type' in params:
            path_params['exposure_type'] = params['exposure_type']

        query_params = {}
        if 'latitude' in params:
            query_params['latitude'] = params['latitude']
        if 'longitude' in params:
            query_params['longitude'] = params['longitude']
        if 'radius' in params:
            query_params['radius'] = params['radius']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='list[Coordinate]',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'),
                                            _preload_content=params.get('_preload_content', True),
                                            _request_timeout=params.get('_request_timeout'),
                                            collection_formats=collection_formats)

    def exposures_exposure_type_dates_get(self, exposure_type, **kwargs):
        """
        Get exposure start date and end date range for exposure type
        Returns exposure start date and end date range for given exposure type

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_exposure_type_dates_get(exposure_type, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str exposure_type: The name of the exposure factor (currently limited to pm25, o3). (required)
        :return: DateRange
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.exposures_exposure_type_dates_get_with_http_info(exposure_type, **kwargs)
        else:
            (data) = self.exposures_exposure_type_dates_get_with_http_info(exposure_type, **kwargs)
            return data

    def exposures_exposure_type_dates_get_with_http_info(self, exposure_type, **kwargs):
        """
        Get exposure start date and end date range for exposure type
        Returns exposure start date and end date range for given exposure type

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_exposure_type_dates_get_with_http_info(exposure_type, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str exposure_type: The name of the exposure factor (currently limited to pm25, o3). (required)
        :return: DateRange
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['exposure_type']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method exposures_exposure_type_dates_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'exposure_type' is set
        if ('exposure_type' not in params) or (params['exposure_type'] is None):
            raise ValueError("Missing the required parameter `exposure_type` when calling `exposures_exposure_type_dates_get`")


        collection_formats = {}

        resource_path = '/exposures/{exposure_type}/dates'.replace('{format}', 'json')
        path_params = {}
        if 'exposure_type' in params:
            path_params['exposure_type'] = params['exposure_type']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='DateRange',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'),
                                            _preload_content=params.get('_preload_content', True),
                                            _request_timeout=params.get('_request_timeout'),
                                            collection_formats=collection_formats)

    def exposures_exposure_type_scores_get(self, exposure_type, start_date, end_date, exposure_point, **kwargs):
        """
        Get exposure score for a given environmental factor at exposure location(s)
        Retrieve the computed exposure score(s) for a given environmental exposure factor, time period, and location(s)

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_exposure_type_scores_get(exposure_type, start_date, end_date, exposure_point, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str exposure_type: The name of the exposure factor (currently limited to pm25, o3). (required)
        :param date start_date: The starting date to obtain exposures for (example 1985-04-12 is April 12th 1985). Currently time of day is ignored. (required)
        :param date end_date: The ending date to obtain exposures for (example 1985-04-13 is April 13th 1985). Currently time of day is ignored. (required)
        :param str exposure_point: A description of the location(s) to retrieve the exposure for. Locaton may be a single geocoordinate (example '35.720278,-79.176389') or a semicomma separated list of geocoord:dayhours giving the start and ending hours on specific days of the week at that location (example '35.720278,-79.176389,Sa0813;35.720278,-79.176389,other') indicates Saturdays from 8am to 1pm is at one location and all other times are at another location. Hours should be in 24 hours time using 2 digits, days of the week should be the first two characters of the day.If the day of the week does not appear then the time periods apply to all days (example '35.720278,-79.176389,0614,35.731944,-78.852778,1424') gives two time periods for all days. If hours do not appear then the time period applies to all hours of the day (example '35.720278,-79.176389,Sa,35.731944,-78.852778,Su'). (required)
        :param str temporal_resolution: The temporal resolution to use for results, should be one of 'hour' or 'day'. Default is 'day'
        :param str score_type: The exposure score type to return. The accepted values vary by exposure factor. For pm25 values are '7dayrisk', '14dayrisk' (NOT COMPLETE)
        :return: list[Exposure]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.exposures_exposure_type_scores_get_with_http_info(exposure_type, start_date, end_date, exposure_point, **kwargs)
        else:
            (data) = self.exposures_exposure_type_scores_get_with_http_info(exposure_type, start_date, end_date, exposure_point, **kwargs)
            return data

    def exposures_exposure_type_scores_get_with_http_info(self, exposure_type, start_date, end_date, exposure_point, **kwargs):
        """
        Get exposure score for a given environmental factor at exposure location(s)
        Retrieve the computed exposure score(s) for a given environmental exposure factor, time period, and location(s)

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_exposure_type_scores_get_with_http_info(exposure_type, start_date, end_date, exposure_point, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str exposure_type: The name of the exposure factor (currently limited to pm25, o3). (required)
        :param date start_date: The starting date to obtain exposures for (example 1985-04-12 is April 12th 1985). Currently time of day is ignored. (required)
        :param date end_date: The ending date to obtain exposures for (example 1985-04-13 is April 13th 1985). Currently time of day is ignored. (required)
        :param str exposure_point: A description of the location(s) to retrieve the exposure for. Locaton may be a single geocoordinate (example '35.720278,-79.176389') or a semicomma separated list of geocoord:dayhours giving the start and ending hours on specific days of the week at that location (example '35.720278,-79.176389,Sa0813;35.720278,-79.176389,other') indicates Saturdays from 8am to 1pm is at one location and all other times are at another location. Hours should be in 24 hours time using 2 digits, days of the week should be the first two characters of the day.If the day of the week does not appear then the time periods apply to all days (example '35.720278,-79.176389,0614,35.731944,-78.852778,1424') gives two time periods for all days. If hours do not appear then the time period applies to all hours of the day (example '35.720278,-79.176389,Sa,35.731944,-78.852778,Su'). (required)
        :param str temporal_resolution: The temporal resolution to use for results, should be one of 'hour' or 'day'. Default is 'day'
        :param str score_type: The exposure score type to return. The accepted values vary by exposure factor. For pm25 values are '7dayrisk', '14dayrisk' (NOT COMPLETE)
        :return: list[Exposure]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['exposure_type', 'start_date', 'end_date', 'exposure_point', 'temporal_resolution', 'score_type']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method exposures_exposure_type_scores_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'exposure_type' is set
        if ('exposure_type' not in params) or (params['exposure_type'] is None):
            raise ValueError("Missing the required parameter `exposure_type` when calling `exposures_exposure_type_scores_get`")
        # verify the required parameter 'start_date' is set
        if ('start_date' not in params) or (params['start_date'] is None):
            raise ValueError("Missing the required parameter `start_date` when calling `exposures_exposure_type_scores_get`")
        # verify the required parameter 'end_date' is set
        if ('end_date' not in params) or (params['end_date'] is None):
            raise ValueError("Missing the required parameter `end_date` when calling `exposures_exposure_type_scores_get`")
        # verify the required parameter 'exposure_point' is set
        if ('exposure_point' not in params) or (params['exposure_point'] is None):
            raise ValueError("Missing the required parameter `exposure_point` when calling `exposures_exposure_type_scores_get`")


        collection_formats = {}

        resource_path = '/exposures/{exposure_type}/scores'.replace('{format}', 'json')
        path_params = {}
        if 'exposure_type' in params:
            path_params['exposure_type'] = params['exposure_type']

        query_params = {}
        if 'start_date' in params:
            query_params['start_date'] = params['start_date']
        if 'end_date' in params:
            query_params['end_date'] = params['end_date']
        if 'exposure_point' in params:
            query_params['exposure_point'] = params['exposure_point']
        if 'temporal_resolution' in params:
            query_params['temporal_resolution'] = params['temporal_resolution']
        if 'score_type' in params:
            query_params['score_type'] = params['score_type']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='list[Exposure]',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'),
                                            _preload_content=params.get('_preload_content', True),
                                            _request_timeout=params.get('_request_timeout'),
                                            collection_formats=collection_formats)

    def exposures_exposure_type_values_get(self, exposure_type, start_date, end_date, exposure_point, **kwargs):
        """
        Get exposure value for a given environmental factor at exposure location(s)
        Retrieve the computed exposure value(s) for a given environmental exposure factor, time period, and location(s)

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_exposure_type_values_get(exposure_type, start_date, end_date, exposure_point, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str exposure_type: The name of the exposure factor (currently limited to pm25, o3). (required)
        :param date start_date: The starting date to obtain exposures for (example 1985-04-12 is April 12th 1985). Currently time of day is ignored. (required)
        :param date end_date: The ending date to obtain exposures for (example 1985-04-13 is April 13th 1985). Currently time of day is ignored. (required)
        :param str exposure_point: A description of the location(s) to retrieve the exposure for. Locaton may be a single geocoordinate (example '35.720278,-79.176389') or a semicomma separated list of geocoord:dayhours giving the start and ending hours on specific days of the week at that location (example '35.720278,-79.176389,Sa0813;35.720278,-79.176389,other') indicates Saturdays from 8am to 1pm is at one location and all other times are at another location. Hours should be in 24 hours time using 2 digits, days of the week should be the first two characters of the day.If the day of the week does not appear then the time periods apply to all days (example '35.720278,-79.176389,0614,35.731944,-78.852778,1424') gives two time periods for all days. If hours do not appear then the time period applies to all hours of the day (example '35.720278,-79.176389,Sa,35.731944,-78.852778,Su'). (required)
        :param str temporal_resolution: The temporal resolution to use for results, should be one of 'hour' or 'day'. Default is 'day'
        :param str statistical_type: The statistic to use for results, should be one of 'max', 'mean', or 'median'. Default is 'max'
        :return: list[Exposure]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.exposures_exposure_type_values_get_with_http_info(exposure_type, start_date, end_date, exposure_point, **kwargs)
        else:
            (data) = self.exposures_exposure_type_values_get_with_http_info(exposure_type, start_date, end_date, exposure_point, **kwargs)
            return data

    def exposures_exposure_type_values_get_with_http_info(self, exposure_type, start_date, end_date, exposure_point, **kwargs):
        """
        Get exposure value for a given environmental factor at exposure location(s)
        Retrieve the computed exposure value(s) for a given environmental exposure factor, time period, and location(s)

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_exposure_type_values_get_with_http_info(exposure_type, start_date, end_date, exposure_point, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str exposure_type: The name of the exposure factor (currently limited to pm25, o3). (required)
        :param date start_date: The starting date to obtain exposures for (example 1985-04-12 is April 12th 1985). Currently time of day is ignored. (required)
        :param date end_date: The ending date to obtain exposures for (example 1985-04-13 is April 13th 1985). Currently time of day is ignored. (required)
        :param str exposure_point: A description of the location(s) to retrieve the exposure for. Locaton may be a single geocoordinate (example '35.720278,-79.176389') or a semicomma separated list of geocoord:dayhours giving the start and ending hours on specific days of the week at that location (example '35.720278,-79.176389,Sa0813;35.720278,-79.176389,other') indicates Saturdays from 8am to 1pm is at one location and all other times are at another location. Hours should be in 24 hours time using 2 digits, days of the week should be the first two characters of the day.If the day of the week does not appear then the time periods apply to all days (example '35.720278,-79.176389,0614,35.731944,-78.852778,1424') gives two time periods for all days. If hours do not appear then the time period applies to all hours of the day (example '35.720278,-79.176389,Sa,35.731944,-78.852778,Su'). (required)
        :param str temporal_resolution: The temporal resolution to use for results, should be one of 'hour' or 'day'. Default is 'day'
        :param str statistical_type: The statistic to use for results, should be one of 'max', 'mean', or 'median'. Default is 'max'
        :return: list[Exposure]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['exposure_type', 'start_date', 'end_date', 'exposure_point', 'temporal_resolution', 'statistical_type']
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method exposures_exposure_type_values_get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'exposure_type' is set
        if ('exposure_type' not in params) or (params['exposure_type'] is None):
            raise ValueError("Missing the required parameter `exposure_type` when calling `exposures_exposure_type_values_get`")
        # verify the required parameter 'start_date' is set
        if ('start_date' not in params) or (params['start_date'] is None):
            raise ValueError("Missing the required parameter `start_date` when calling `exposures_exposure_type_values_get`")
        # verify the required parameter 'end_date' is set
        if ('end_date' not in params) or (params['end_date'] is None):
            raise ValueError("Missing the required parameter `end_date` when calling `exposures_exposure_type_values_get`")
        # verify the required parameter 'exposure_point' is set
        if ('exposure_point' not in params) or (params['exposure_point'] is None):
            raise ValueError("Missing the required parameter `exposure_point` when calling `exposures_exposure_type_values_get`")


        collection_formats = {}

        resource_path = '/exposures/{exposure_type}/values'.replace('{format}', 'json')
        path_params = {}
        if 'exposure_type' in params:
            path_params['exposure_type'] = params['exposure_type']

        query_params = {}
        if 'start_date' in params:
            query_params['start_date'] = params['start_date']
        if 'end_date' in params:
            query_params['end_date'] = params['end_date']
        if 'exposure_point' in params:
            query_params['exposure_point'] = params['exposure_point']
        if 'temporal_resolution' in params:
            query_params['temporal_resolution'] = params['temporal_resolution']
        if 'statistical_type' in params:
            query_params['statistical_type'] = params['statistical_type']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='list[Exposure]',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'),
                                            _preload_content=params.get('_preload_content', True),
                                            _request_timeout=params.get('_request_timeout'),
                                            collection_formats=collection_formats)

    def exposures_get(self, **kwargs):
        """
        Get list of exposure types
        Returns a list of all available exposure types

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_get(callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :return: list[ExposureType]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.exposures_get_with_http_info(**kwargs)
        else:
            (data) = self.exposures_get_with_http_info(**kwargs)
            return data

    def exposures_get_with_http_info(self, **kwargs):
        """
        Get list of exposure types
        Returns a list of all available exposure types

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.exposures_get_with_http_info(callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :return: list[ExposureType]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []
        all_params.append('callback')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method exposures_get" % key
                )
            params[key] = val
        del params['kwargs']


        collection_formats = {}

        resource_path = '/exposures'.replace('{format}', 'json')
        path_params = {}

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type([])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='list[ExposureType]',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'),
                                            _preload_content=params.get('_preload_content', True),
                                            _request_timeout=params.get('_request_timeout'),
                                            collection_formats=collection_formats)



# coding: utf-8

"""
    Environmental Exposures API

    Environmental Exposures API

    OpenAPI spec version: 1.0.0
    Contact: stealey@renci.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class Exposure(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, exposure_type=None, start_time=None, end_time=None, value=None, units=None, latitude=None, longitude=None):
        """
        Exposure - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'exposure_type': 'str',
            'start_time': 'datetime',
            'end_time': 'datetime',
            'value': 'str',
            'units': 'str',
            'latitude': 'str',
            'longitude': 'str'
        }

        self.attribute_map = {
            'exposure_type': 'exposure_type',
            'start_time': 'start_time',
            'end_time': 'end_time',
            'value': 'value',
            'units': 'units',
            'latitude': 'latitude',
            'longitude': 'longitude'
        }

        self._exposure_type = exposure_type
        self._start_time = start_time
        self._end_time = end_time
        self._value = value
        self._units = units
        self._latitude = latitude
        self._longitude = longitude


    @property
    def exposure_type(self):
        """
        Gets the exposure_type of this Exposure.


        :return: The exposure_type of this Exposure.
        :rtype: str
        """
        return self._exposure_type

    @exposure_type.setter
    def exposure_type(self, exposure_type):
        """
        Sets the exposure_type of this Exposure.


        :param exposure_type: The exposure_type of this Exposure.
        :type: str
        """
        if exposure_type is None:
            raise ValueError("Invalid value for `exposure_type`, must not be `None`")

        self._exposure_type = exposure_type

    @property
    def start_time(self):
        """
        Gets the start_time of this Exposure.


        :return: The start_time of this Exposure.
        :rtype: datetime
        """
        return self._start_time

    @start_time.setter
    def start_time(self, start_time):
        """
        Sets the start_time of this Exposure.


        :param start_time: The start_time of this Exposure.
        :type: datetime
        """

        self._start_time = start_time

    @property
    def end_time(self):
        """
        Gets the end_time of this Exposure.


        :return: The end_time of this Exposure.
        :rtype: datetime
        """
        return self._end_time

    @end_time.setter
    def end_time(self, end_time):
        """
        Sets the end_time of this Exposure.


        :param end_time: The end_time of this Exposure.
        :type: datetime
        """

        self._end_time = end_time

    @property
    def value(self):
        """
        Gets the value of this Exposure.


        :return: The value of this Exposure.
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """
        Sets the value of this Exposure.


        :param value: The value of this Exposure.
        :type: str
        """

        self._value = value

    @property
    def units(self):
        """
        Gets the units of this Exposure.


        :return: The units of this Exposure.
        :rtype: str
        """
        return self._units

    @units.setter
    def units(self, units):
        """
        Sets the units of this Exposure.


        :param units: The units of this Exposure.
        :type: str
        """

        self._units = units

    @property
    def latitude(self):
        """
        Gets the latitude of this Exposure.


        :return: The latitude of this Exposure.
        :rtype: str
        """
        return self._latitude

    @latitude.setter
    def latitude(self, latitude):
        """
        Sets the latitude of this Exposure.


        :param latitude: The latitude of this Exposure.
        :type: str
        """

        self._latitude = latitude

    @property
    def longitude(self):
        """
        Gets the longitude of this Exposure.


        :return: The longitude of this Exposure.
        :rtype: str
        """
        return self._longitude

    @longitude.setter
    def longitude(self, longitude):
        """
        Sets the longitude of this Exposure.


        :param longitude: The longitude of this Exposure.
        :type: str
        """

        self._longitude = longitude

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""
    Environmental Exposures API

    Environmental Exposures API

    OpenAPI spec version: 1.0.0
    Contact: stealey@renci.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from __future__ import absolute_import

# import models into sdk package
from .models.coordinate import Coordinate
from .models.date_range import DateRange
from .models.exposure import Exposure
from .models.exposure_type import ExposureType

# import apis into sdk package
from .apis.default_api import DefaultApi

# import ApiClient
from .api_client import ApiClient

from .configuration import Configuration

configuration = Configuration()

__all__ = [ 'models', 'apis' ]




""" This part is generic provenance infrastructure """

import importlib
import json 
import prov.model as prov
from datetime import datetime
from datetime import date
import re
import threading
from threading import current_thread
import traceback
import uuid

class QueryResponseProvenance(object):
    """ Abstract W3C PROV model of the provenance of components of a complex query. """
    def __init__(self, default_ns, namespaces=[]):
        """ Specify a default namespace and associated sub namespaces """
        self.document = prov.ProvDocument ()
        self.default_ns = default_ns
        self.document.set_default_namespace (self.default_ns)
        self.namespaces = namespaces
        self.subspaces = {}
        for namespace in self.namespaces:
            self.subspaces[namespace] = self.add_namespace (self.default_ns, namespace)
    def add_namespace (self, root, qualifier):
        subspace = "{0}{1}".format (root, qualifier)
        self.document.add_namespace (qualifier, subspace)
        return subspace
    def add_entity (self, name, tuples):
        self.document.entity (name, tuples)
    def add_data_source (self, name, entity):
        self.entity (name, entity.to_tuple ())
    def add_algorithm (self, name, start, end=None):
        #assert name in self.namespaces, "Name must be in list of namespaces"
        attributes = [ ( "TODO:id", get_provenance_id () ) ]
        if end:
            self.document.activity (name, start, end, other_attributes=attributes)
        else:
            self.document.activity (name, start, other_attributes=attributes)
    def get_time (self):
        return datetime.now ().strftime ("%Y-%m-%dT%H:%M:%S")
    def __str__(self):
        return self.__repr__()
    def __repr__(self):
        return self.document.get_provn ()
 
class AbstractEntity(object):
    def __init__(self, type, namespace, attributes=[]):
        self.attributes = []
        self.namespace = namespace
        self.attr_keys = {}
        self.add_attribute (prov.PROV_TYPE, type)
        for a in attributes:
            assert len(a) == 2, "Attribute components must be len==2 arrays"
            self.add_attribute (a[0], a[1])
    def add_attribute (self, iri, value):
        key = '{0}@{1}'.format (iri, value)
        if not key in self.attr_keys:
            self.attributes.append ((iri, value))
            self.attr_keys[key] = 1
    def to_tuple (self):
        return tuple(self.attributes)
  
""" DataLake Support - encapsulate provenance behaviors specific to
    specific classes of data resources """
 
class DataLakeProvenance(QueryResponseProvenance):
    PREFIX = re.compile ('^prefix ', re.I)
    def __init__(self, default_ns, namespaces=[]):
        QueryResponseProvenance.__init__(self, default_ns, namespaces)
 
    def parse_sparql (self, text, source_map, type="data"):
        sources = {}
        for line in text.split ('\n'):
            line = ' '.join (line.strip ().split ())
            match = self.PREFIX.match (line)
            if match is None:
                continue
            if True:
                parts = line.strip().split (' ')
                if len(parts) >= 3:
                    iri = parts[2].strip ()
                    e = None
                    for k, v in source_map.items ():
                        if k in iri:
                            if v in sources:
                                e = sources[v]
                            else:
                                e = AbstractEntity (type, '{0}{1}'.format (self.default_ns, v))
                                sources[v] = e
                            e.add_attribute ('TODO:id', get_provenance_id ())
                            e.add_attribute ('src', v)
                    if e is None:
                        e = AbstractEntity (type, 'http://purl.data.org/')
                        e.add_attribute ('TODO:id', get_provenance_id ())
                        e.add_attribute ('src', iri)
                        self.add_entity ('data', e.to_tuple ())
        for k, v in sources.items ():
            self.add_entity ('data', v.to_tuple ())
 
""" And now we get really specific about one data lake, the green translator. """
 
class GreenTranslatorProvenance (DataLakeProvenance):
    def __init__(self):
        DataLakeProvenance.__init__(
            self,
            default_ns = 'http://purl.translator.org/prov/',
            namespaces = [
                'expo', 'biochem', 'TODO',                          # data sources
                'expo.pm25-o3', 'clinical.med.prescribed', 'blazegraph' # algorithms / assumptions
            ])
    def parse_sparql (self, query):
        super(GreenTranslatorProvenance, self).parse_sparql (
            query,
            source_map = {
                '<http://chem2bio2rdf.org/ctd/' : 'c2b2r.ctd',
                'GO_' : 'GO',
                '<http://chem2bio2rdf.org/drugbank' : 'c2b2r.drugbank',
                'monarch' : 'monarch'
            },
            type="biochem:data")

class ProvenanceQuery(object):

    """ Query with provenance """
    def __init__(self):
        self.provenance = GreenTranslatorProvenance ()

    def get_prov (self):
        return json.loads (self.provenance.document.serialize(format='json'))

    def prov_json (self):
        return json.dumps (json.loads (self.provenance.document.serialize(format='json')),
                           indent=2)

threadProvenance = threading.local()

def get_thread_prov_container (initialize=False):
    result = None
    initialized = getattr(threadProvenance, 'initialized', None)
    if initialized is None:
        if initialize:
            print("Initializing thread provenance @thread: {0}".format (current_thread().name))
            threadProvenance.initialized = True
            result = threadProvenance
        else:
            raise "Thread provenance not initialized and initialize set to false @thread: {0}".format (current_thread().name)
    else:
        #print ("Provenance already initialized @thread: {0}".format (current_thread().name))
        result = threadProvenance
    return result
def register_provenance (p):
    container = get_thread_prov_container (initialize=True)
    container.provenance = p
    container.uuid = "http://id.org/{0}".format (uuid.uuid4())
def get_provenance ():
    return get_thread_prov_container ().provenance
def get_provenance_id ():
    return get_thread_prov_container ().uuid
def unregister_provenance ():
    container = get_thread_prov_container()
    container.provenance = None
    container.uuid = None

provenance_conf = {

    "expo_get_scores" : {
        "type" : "expo:score",
        "sources" : {
            "expo" : "http://purl.expo.org/unc/cdwh",
            "expo" : "http://purl.expo.org/unc/ie"
        }
    },
    "expo_get_values" : {
        "type" : "expo:value",
        "sources" : {
            "expo" : "http://purl.expo.org/unc/cdwh",
            "expo" : "http://purl.expo.org/unc/ie"
        }
    },

    # BioChem

    "execute_query" : {
        "handler" : {
            "module" : "greentranslator.provenance",
            "function_name" : "parse_sparql_query_provenance"
        },
        "type" : "biochem:sparql",
        "sources" : {
            "biochem" : "http://purl.biochem.org/c2b2r"
        }
    },

    "get_exposure_conditions" : {
        "type" : "biochem:exposures",
        "sources" : {
            "biochem" : "http://purl.biochem.org/c2b2r"
        }
    },
    "get_genes_pathways_by_disease" : {
        "type" : "biochem:genes_pathways_by_disease",
        "sources" : {
            "biochem" : "http://purl.biochem.org/c2b2r"
        }
    },
    "get_drugs_by_condition" : {
        "type" : "biochem:get_drugs_by_condition",
        "sources" : {
            "biochem" : "http://purl.biochem.org/c2b2r"
        }
    }
}

def parse_sparql_query_provenance (prov_obj, function, args, kwargs, start, end):
    prov_obj.add_algorithm ('blazegraph', start, end)
    prov_obj.parse_sparql (args[1])

def provenance (data_source=None):
    def provenance_aspect (function):
        def wrapper (*args, **kwargs):
            func_name = function.__name__
            a_self = args[0]
            if isinstance (a_self, ProvenanceQuery):
                prov_obj = a_self.provenance                
                register_provenance (a_self.provenance)
            else:
                prov_obj = get_provenance ()

            start = prov_obj.get_time ()
            val = function(*args, **kwargs)
            end = prov_obj.get_time ()
            
            try:
                if prov_obj:
                    for op, desc in provenance_conf.items ():
                        if op == func_name:
                            prov_obj.add_algorithm (func_name, start, end)
                            e = AbstractEntity (func_name, 'TODO:schema')
                            e.add_attribute ('TODO:id', get_provenance_id ())
                            e.add_attribute (desc['type'], '{0}({1}{2})'.format (func_name, args[1:], kwargs))
                            prov_obj.add_entity ("TODO:schema", e.to_tuple ())

                            if 'handler' in desc:
                                handler = desc['handler']
                                module = importlib.import_module (handler['module'])
                                handler_func = getattr (module, handler['function_name'])
                                handler_func (prov_obj, function, args, kwargs, start, end)

                            for source, iri in desc['sources'].items ():
                                e = AbstractEntity (source, iri)
                                e.add_attribute ('src', source)
                                e.add_attribute ('TODO:id', get_provenance_id ())
                                prov_obj.add_entity (source, e.to_tuple ())
            except:
                traceback.print_exc ()
                print ("Error recording provenance")

            unregister_provenance ()
            return val            
        return wrapper
    return provenance_aspect

